@@ -13,54 +13,70 @@ __doc__ = """\
 3. Windows 要安装 WinFsp： https://github.com/winfsp/winfsp
 """
 
-epilog = """\
+epilog = """---------- 使用帮助 ----------
+
 1. 隐藏所有 *.mkv 文件
 
 .. code: console
 
-    python -m alist fuse --show-predicate '*.mkv'
-
+    python-alist fuse --show-predicate '*.mkv'
 
 2. 只显示所有文件夹和 *.mkv 文件
 
 .. code: console
 
-    python -m alist fuse --show-predicate '* !/**/ !*.mkv'
+    python-alist fuse --show-predicate '* !/**/ !*.mkv'
 
-3. 把所有视频、音频、图片显示为 STRM 文件
+或者
 
 .. code: console
 
-    python -m alist fuse \\
+    python-alist fuse \\
         --show-predicate-type expr \\
-        --show-predicate 'path.is_dir() or path.media_type.startswith(("video/", "audio/", "image/")) or path.suffix in (".nfo", ".ass", ".ssa", ".srt", ".idx", ".sub", ".vtt", ".smi")' \
+        --show-predicate 'path.is_dir() or path.suffix.lower() == ".mkv"'
+
+3. 把所有视频、音频显示为 .strm 文件，显示图片、字幕和 .nfo 文件
+
+.. code: console
+
+    python-alist fuse \\
         --strm-predicate-type expr \\
-        --strm-predicate 'path.media_type.startswith(("video/", "audio/")) and path.suffix != ".ass"'
+        --strm-predicate '(
+            path.media_type.startswith(("video/", "audio/")) and 
+            path.suffix.lower() != ".ass"
+        )' \\
+        --show-predicate-type expr \\
+        --show-predicate '(
+            path.is_dir() or 
+            path.media_type.startswith("image/") or 
+            path.suffix.lower() in (".nfo", ".ass", ".ssa", ".srt", ".idx", ".sub", ".txt", ".vtt", ".smi")
+        )'
 
 4. 把缓存保存到本地的 dbm 文件
 
 .. code: console
 
-    python -m alist fuse '
+    python-alist fuse '
     import shelve
     cache = shelve.open("cache")'
+
+5. 自定义生成的 strm，例如把 base-url 设置为 http://my.302.server
+
+    python-alist fuse --strm-predicate '*' --custom-strm 'http://my.302.server'
 """
 
 if __name__ == "__main__":
     from argparse import ArgumentParser, RawTextHelpFormatter
-    from fuser import AlistFuseOperations # type: ignore
-    from util.log import logger # type: ignore
-    from util.predicate import make_predicate # type: ignore
+    from pathlib import Path
+    from sys import path
 
-    parser = ArgumentParser(description=__doc__, formatter_class=RawTextHelpFormatter)
+    path[0] = str(Path(__file__).parents[3])
+    parser = ArgumentParser(description=__doc__, epilog=epilog, formatter_class=RawTextHelpFormatter)
 else:
     from argparse import RawTextHelpFormatter
     from ..init import subparsers
-    from .fuser import AlistFuseOperations
-    from .util.log import logger
-    from .util.predicate import make_predicate
 
-    parser = subparsers.add_parser("fuse", description=__doc__, formatter_class=RawTextHelpFormatter)
+    parser = subparsers.add_parser("fuse", description=__doc__, epilog=epilog, formatter_class=RawTextHelpFormatter)
 
 
 def main(args):
@@ -69,6 +85,11 @@ def main(args):
         print(".".join(map(str, __version__)))
         raise SystemExit(0)
 
+    from alist.cmd.fuse.util.fuser import AlistFuseOperations
+    from alist.cmd.fuse.util.log import logger
+    from alist.cmd.fuse.util.predicate import make_predicate
+    from alist.cmd.fuse.util.strm import parse as make_strm_converter
+
     mount_point = args.mount_point
     if not mount_point:
         from uuid import uuid4
@@ -77,23 +98,32 @@ def main(args):
     import logging
 
     log_level = args.log_level
-    if isinstance(log_level, str):
-        try:
-            log_level = int(log_level)
-        except ValueError:
-            log_level = getattr(logging, log_level.upper(), logging.NOTSET)
+    if log_level.isascii() and log_level.isdecimal():
+        log_level = int(log_level)
+    else:
+        log_level = getattr(logging, log_level.upper(), logging.NOTSET)
     logger.setLevel(log_level)
 
     import re
 
-    predicate = args.show_predicate
-    if predicate:
+    if predicate := args.show_predicate:
         predicate = make_predicate(predicate, {"re": re}, type=args.show_predicate_type)
 
-    strm_predicate = args.strm_predicate
-    if strm_predicate:
+    if strm_predicate := args.strm_predicate:
         strm_predicate = make_predicate(strm_predicate, {"re": re}, type=args.strm_predicate_type)
 
+    strm_make = None
+    if custom_strm := args.custom_strm:
+        custom_strm_type = args.custom_strm_type
+        if custom_strm_type == "file":
+            from pathlib import Path
+            custom_strm = custom_strm(custom_strm)
+        strm_make = make_strm_converter(
+            custom_strm, 
+            {"re": re, "token": args.token}, 
+            code_type=custom_strm_type, 
+        )
+
     from re import compile as re_compile
 
     CRE_PAT_IN_STR = re_compile(r"[^\\ ]*(?:\\(?s:.)[^\\ ]*)*")
@@ -134,13 +164,17 @@ def main(args):
 
     # https://code.google.com/archive/p/macfuse/wikis/OPTIONS.wiki
     AlistFuseOperations(
-        args.origin, 
-        args.username, 
-        args.password, 
+        origin=args.origin, 
+        username=args.username, 
+        password=args.password, 
+        token=args.token, 
+        base_dir=args.base_dir, 
         cache=cache, 
+        max_readdir_workers=args.max_readdir_workers, 
+        max_readdir_cooldown=args.max_readdir_cooldown, 
         predicate=predicate, 
         strm_predicate=strm_predicate, 
-        max_readdir_workers=args.max_readdir_workers, 
+        strm_make=strm_make, 
         direct_open_names=direct_open_names, 
         direct_open_exes=direct_open_exes, 
     ).run(
@@ -157,9 +191,15 @@ parser.add_argument("mount_point", nargs="?", help="挂载路径")
 parser.add_argument("-o", "--origin", default="http://localhost:5244", help="alist 服务器地址，默认 http://localhost:5244")
 parser.add_argument("-u", "--username", default="", help="用户名，默认为空")
 parser.add_argument("-p", "--password", default="", help="密码，默认为空")
+parser.add_argument("-t", "--token", default="", help="token，用于给链接做签名，默认为空")
+parser.add_argument("-bd", "--base-dir", default="/", help="挂载的目录，默认为 '/'")
 parser.add_argument(
-    "-m", "--max-readdir-workers", default=8, type=int, 
-    help="读取目录的文件列表的最大的并发线程数，默认值是 8，等于 0 则自动确定，小于 0 则不限制", 
+    "-mr", "--max-readdir-workers", default=5, type=int, 
+    help="读取目录的文件列表的最大的并发线程数，默认值是 5，等于 0 则自动确定，小于 0 则不限制", 
+)
+parser.add_argument(
+    "-mc", "--max-readdir-cooldown", default=30, type=float, 
+    help="读取目录的文件列表的冷却时间（单位：秒），在冷却时间内会直接返回缓存的数据（避免更新），默认值是 30，小于等于 0 则不限制", 
 )
 parser.add_argument("-p1", "--show-predicate", help="断言，当断言的结果为 True 时，文件或目录会被显示")
 parser.add_argument(
@@ -171,13 +211,13 @@ parser.add_argument(
     - ignore-file  接受一个文件路径，包含 gitignore 的配置文本（一行一个），在文件路径上执行模式匹配，匹配成功则断言为 False
                    NOTE: https://git-scm.com/docs/gitignore#_pattern_format
     - expr         表达式，会注入一个名为 path 的 alist.AlistPath 对象
-    - re           正则表达式，如果文件的名字匹配此模式，则断言为 True
     - lambda       lambda 函数，接受一个 alist.AlistPath 对象作为参数
     - stmt         语句，当且仅当不抛出异常，则视为 True，会注入一个名为 path 的 alist.AlistPath 对象
-    - code         代码，运行后需要在它的全局命名空间中生成一个 check 或 predicate 函数用于断言，接受一个 alist.AlistPath 对象作为参数
-    - path         代码的路径，运行后需要在它的全局命名空间中生成一个 check 或 predicate 函数用于断言，接受一个 alist.AlistPath 对象作为参数
+    - module       代码，运行后需要在它的全局命名空间中生成一个 check 或 predicate 函数用于断言，接受一个 alist.AlistPath 对象作为参数
+    - file         代码的路径，运行后需要在它的全局命名空间中生成一个 check 或 predicate 函数用于断言，接受一个 alist.AlistPath 对象作为参数
+    - re           正则表达式，如果文件的名字匹配此模式，则断言为 True
 """)
-parser.add_argument("-p2", "--strm-predicate", help="strm 断言，当断言的结果为 True 时，文件会被显示为带有 .strm 后缀的文本文件，打开后是链接")
+parser.add_argument("-p2", "--strm-predicate", help="strm 断言（优先级高于 -p1/--show-predicate），当断言的结果为 True 时，文件会被显示为带有 .strm 后缀的文本文件，打开后是链接")
 parser.add_argument(
     "-t2", "--strm-predicate-type", default="filter", 
     choices=("filter", "filter-file", "expr", "re", "lambda", "stmt", "code", "path"), 
@@ -187,11 +227,28 @@ parser.add_argument(
     - filter-file  接受一个文件路径，包含 gitignore 的配置文本（一行一个），在文件路径上执行模式匹配，匹配成功则断言为 True
                    请参考：https://git-scm.com/docs/gitignore#_pattern_format
     - expr         表达式，会注入一个名为 path 的 alist.AlistPath 对象
-    - re           正则表达式，如果文件的名字匹配此模式，则断言为 True
     - lambda       lambda 函数，接受一个 alist.AlistPath 对象作为参数
     - stmt         语句，当且仅当不抛出异常，则视为 True，会注入一个名为 path 的 alist.AlistPath 对象
-    - code         代码，运行后需要在它的全局命名空间中生成一个 check 或 predicate 函数用于断言，接受一个 alist.AlistPath 对象作为参数
-    - path         代码的路径，运行后需要在它的全局命名空间中生成一个 check 或 predicate 函数用于断言，接受一个 alist.AlistPath 对象作为参数
+    - module       代码，运行后需要在它的全局命名空间中生成一个 check 或 predicate 函数用于断言，接受一个 alist.AlistPath 对象作为参数
+    - file         代码的路径，运行后需要在它的全局命名空间中生成一个 check 或 predicate 函数用于断言，接受一个 alist.AlistPath 对象作为参数
+    - re           正则表达式，如果文件的名字匹配此模式，则断言为 True
+""")
+parser.add_argument("-cs", "--custom-strm", help="自定义 strm 的内容")
+parser.add_argument(
+    "-ct", "--custom-strm-type", default="base-url", 
+    choices=("filter", "filter-file", "expr", "re", "lambda", "stmt", "code", "path"), 
+    help="""自定义 strm 的操作类型，默认值 'base-url'，以返回值作为 strm 中的链接，如果报错，则不生成 strm 文件
+    - base-url  提供一个 base-url，用来拼接（相对）路径
+    - expr      表达式，可从命名空间访问到一个名为 path 的 alist.AlistPath 对象
+    - fstring   视为 fstring，可从命名空间访问到一个名为 path 的 alist.AlistPath 对象
+    - lambda    lambda 函数，接受一个 alist.AlistPath 对象作为参数
+    - stmt      语句，可从命名空间访问到一个名为 path 的 alist.AlistPath 对象，最后要产生一个名为 url 的变量到本地命名空间
+    - module    作为一个模块被夹在，运行后需要在它的全局命名空间中生成一个 run 或 convert 函数，接受一个 alist.AlistPath 对象作为参数
+    - file      文件路径，会被作为模块加载执行，运行后需要在它的全局命名空间中生成一个 run 或 convert 函数，接受一个 alist.AlistPath 对象作为参数
+    - resub     正则表达式，语法同 sed，格式为 /pattern/replacement/flag，用来对生成的链接进行搜索替换
+上面的各个类型，都会注入几个全局变量
+    - re      正则表达式模块
+    - token   Alist 的 token，经命令行传入
 """)
 parser.add_argument(
     "-dn", "--direct-open-names", 
@@ -232,7 +289,7 @@ Reference:
     - https://docs.python.org/3/library/collections.abc.html#collections-abstract-base-classes
 """)
 parser.add_argument("-d", "--debug", action="store_true", help="调试模式，输出更多信息")
-parser.add_argument("-l", "--log-level", default=0, help=f"指定日志级别，可以是数字或名称，不传此参数则不输出日志，默认值: 0 (NOTSET)")
+parser.add_argument("-l", "--log-level", default="NOTSET", help=f"指定日志级别，可以是数字或名称，不传此参数则不输出日志，默认值: 'NOTSET'")
 parser.add_argument("-b", "--background", action="store_true", help="后台运行")
 parser.add_argument("-s", "--nothreads", action="store_true", help="不用多线程")
 parser.add_argument("--allow-other", action="store_true", help="允许 other 用户（也即不是 user 和 group）")
